@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef ANNOTATEDITEMDELEGATE_H
-#define ANNOTATEDITEMDELEGATE_H
+#pragma once
 
 #include "utils_global.h"
 
@@ -35,8 +34,8 @@ namespace Utils {
 class QTCREATOR_UTILS_EXPORT AnnotatedItemDelegate : public QStyledItemDelegate
 {
 public:
-    AnnotatedItemDelegate(QObject *parent = 0);
-    virtual ~AnnotatedItemDelegate();
+    AnnotatedItemDelegate(QObject *parent = nullptr);
+    ~AnnotatedItemDelegate() override;
 
     void setAnnotationRole(int role);
     int annotationRole() const;
@@ -45,10 +44,10 @@ public:
     const QString &delimiter() const;
 
 protected:
-    virtual void paint(QPainter *painter,
+    void paint(QPainter *painter,
                        const QStyleOptionViewItem &option,
-                       const QModelIndex &index) const;
-    virtual QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
+                       const QModelIndex &index) const override;
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
 
 private:
     int m_annotationRole;
@@ -56,5 +55,3 @@ private:
 };
 
 } // Utils
-
-#endif // ANNOTATEDITEMDELEGATE_H
