@@ -34,6 +34,7 @@
 #include <QFileInfo>
 #include <QCommonStyle>
 #include <QStyleOption>
+#include <QWindow>
 #include <qmath.h>
 
 // Clamps float color values within (0, 255)
@@ -64,6 +65,18 @@ QColor StyleHelper::mergedColors(const QColor &colorA, const QColor &colorB, int
     return tmp;
 }
 
+QColor StyleHelper::alphaBlendedColors(const QColor &colorA, const QColor &colorB)
+{
+    const int alpha = colorB.alpha();
+    const int antiAlpha = 255 - alpha;
+
+    return QColor(
+                (colorA.red() * antiAlpha + colorB.red() * alpha) / 255,
+                (colorA.green() * antiAlpha + colorB.green() * alpha) / 255,
+                (colorA.blue() * antiAlpha + colorB.blue() * alpha) / 255
+                );
+}
+
 qreal StyleHelper::sidebarFontSize()
 {
     return HostOsInfo::isMacHost() ? 10 : 7.5;
@@ -71,7 +84,7 @@ qreal StyleHelper::sidebarFontSize()
 
 QColor StyleHelper::notTooBrightHighlightColor()
 {
-    QColor highlightColor = qApp->palette().highlight().color();
+    QColor highlightColor = QApplication::palette().highlight().color();
     if (0.5 * highlightColor.saturationF() + 0.75 - highlightColor.valueF() < 0)
         highlightColor.setHsvF(highlightColor.hsvHueF(), 0.1 + highlightColor.saturationF() * 2.0, highlightColor.valueF());
     return highlightColor;
@@ -81,16 +94,14 @@ QColor StyleHelper::notTooBrightHighlightColor()
 QPalette StyleHelper::sidebarFontPalette(const QPalette &original)
 {
     QPalette palette = original;
-    palette.setColor(QPalette::Active, QPalette::Text, panelTextColor());
-    palette.setColor(QPalette::Active, QPalette::WindowText, panelTextColor());
-    palette.setColor(QPalette::Inactive, QPalette::Text, panelTextColor().darker());
-    palette.setColor(QPalette::Inactive, QPalette::WindowText, panelTextColor().darker());
+    const QColor textColor = creatorTheme()->color(Theme::ProgressBarTitleColor);
+    palette.setColor(QPalette::WindowText, textColor);
+    palette.setColor(QPalette::Text, textColor);
     return palette;
 }
 
 QColor StyleHelper::panelTextColor(bool lightColored)
 {
-    //qApp->palette().highlightedText().color();
     if (!lightColored)
         return Qt::white;
     else
@@ -103,10 +114,10 @@ QColor StyleHelper::m_requestedBaseColor;
 
 QColor StyleHelper::baseColor(bool lightColored)
 {
-    if (!lightColored)
-        return m_baseColor;
-    else
-        return m_baseColor.lighter(230);
+    static const QColor windowColor = QApplication::palette().color(QPalette::Window);
+    static const bool windowColorAsBase = creatorTheme()->flag(Theme::WindowColorAsBase);
+
+    return (lightColored || windowColorAsBase) ? windowColor : m_baseColor;
 }
 
 QColor StyleHelper::highlightColor(bool lightColored)
@@ -141,6 +152,14 @@ QColor StyleHelper::borderColor(bool lightColored)
     return result;
 }
 
+QColor StyleHelper::toolBarBorderColor()
+{
+    const QColor base = baseColor();
+    return QColor::fromHsv(base.hue(),
+                           base.saturation() ,
+                           clamp(base.value() * 0.80f));
+}
+
 // We try to ensure that the actual color used are within
 // reasonalbe bounds while generating the actual baseColor
 // from the users request.
@@ -315,7 +334,8 @@ void StyleHelper::drawArrow(QStyle::PrimitiveElement element, QPainter *painter,
         if (!enabled) {
             drawCommonStyleArrow(image.rect(), creatorTheme()->color(Theme::IconsDisabledColor));
         } else {
-            drawCommonStyleArrow(image.rect().translated(0, devicePixelRatio), toolBarDropShadowColor());
+            if (creatorTheme()->flag(Theme::ToolBarIconShadow))
+                drawCommonStyleArrow(image.rect().translated(0, devicePixelRatio), toolBarDropShadowColor());
             drawCommonStyleArrow(image.rect(), creatorTheme()->color(Theme::IconsBaseColor));
         }
         painter.end();
@@ -372,7 +392,9 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
                                      QPainter *p, QIcon::Mode iconMode, int dipRadius, const QColor &color, const QPoint &dipOffset)
 {
     QPixmap cache;
-    QString pixmapName = QString::fromLatin1("icon %0 %1 %2").arg(icon.cacheKey()).arg(iconMode).arg(rect.height());
+    const int devicePixelRatio = p->device()->devicePixelRatio();
+    QString pixmapName = QString::fromLatin1("icon %0 %1 %2 %3")
+            .arg(icon.cacheKey()).arg(iconMode).arg(rect.height()).arg(devicePixelRatio);
 
     if (!QPixmapCache::find(pixmapName, cache)) {
         // High-dpi support: The in parameters (rect, radius, offset) are in
@@ -380,9 +402,8 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
         // return a high-dpi pixmap, which will in that case have a devicePixelRatio
         // different than 1. The shadow drawing caluculations are done in device
         // pixels.
-        QWindow *window = QApplication::allWidgets().first()->windowHandle();
+        QWindow *window = dynamic_cast<QWidget*>(p->device())->window()->windowHandle();
         QPixmap px = icon.pixmap(window, rect.size(), iconMode);
-        int devicePixelRatio = qCeil(px.devicePixelRatio());
         int radius = dipRadius * devicePixelRatio;
         QPoint offset = dipOffset * devicePixelRatio;
         cache = QPixmap(px.size() + QSize(radius * 2, radius * 2));
@@ -390,10 +411,11 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
 
         QPainter cachePainter(&cache);
         if (iconMode == QIcon::Disabled) {
-            const bool hasDisabledState = icon.availableSizes(QIcon::Disabled).contains(px.size());
+            const bool hasDisabledState =
+                    icon.availableSizes().count() == icon.availableSizes(QIcon::Disabled).count();
             if (!hasDisabledState)
                 px = disabledSideBarIcon(icon.pixmap(window, rect.size()));
-        } else {
+        } else if (creatorTheme()->flag(Theme::ToolBarIconShadow)) {
             // Draw shadow
             QImage tmp(px.size() + QSize(radius * 2, radius * 2 + 1), QImage::Format_ARGB32_Premultiplied);
             tmp.fill(Qt::transparent);
@@ -429,6 +451,7 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
 
         // Draw the actual pixmap...
         cachePainter.drawPixmap(QRect(QPoint(radius, radius) + offset, QSize(px.width(), px.height())), px);
+        cachePainter.end();
         cache.setDevicePixelRatio(devicePixelRatio);
         QPixmapCache::insert(pixmapName, cache);
     }
