@@ -32,19 +32,21 @@
 #include "testsettings.h"
 #include "testtreemodel.h"
 #include "testcodeparser.h"
+#include "testeditormark.h"
 
+#include <aggregation/aggregate.h>
+#include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/coreicons.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/find/basetextfind.h>
 #include <coreplugin/find/itemviewfind.h>
 #include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
-
+#include <projectexplorer/buildmanager.h>
 #include <projectexplorer/projectexplorer.h>
-
 #include <texteditor/texteditor.h>
-
 #include <utils/theme/theme.h>
+#include <utils/utilsicons.h>
 
 #include <QApplication>
 #include <QClipboard>
@@ -54,6 +56,7 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QScrollBar>
+#include <QStackedWidget>
 #include <QToolButton>
 #include <QVBoxLayout>
 
@@ -77,16 +80,15 @@ void ResultsTreeView::keyPressEvent(QKeyEvent *event)
 
 TestResultsPane::TestResultsPane(QObject *parent) :
     Core::IOutputPane(parent),
-    m_context(new Core::IContext(this)),
-    m_wasVisibleBefore(false),
-    m_autoScroll(false),
-    m_testRunning(false)
+    m_context(new Core::IContext(this))
 {
-    m_outputWidget = new QWidget;
+    m_outputWidget = new QStackedWidget;
+    QWidget *visualOutputWidget = new QWidget;
+    m_outputWidget->addWidget(visualOutputWidget);
     QVBoxLayout *outputLayout = new QVBoxLayout;
     outputLayout->setMargin(0);
     outputLayout->setSpacing(0);
-    m_outputWidget->setLayout(outputLayout);
+    visualOutputWidget->setLayout(outputLayout);
 
     QPalette pal;
     pal.setColor(QPalette::Window,
@@ -106,10 +108,13 @@ TestResultsPane::TestResultsPane(QObject *parent) :
 
     outputLayout->addWidget(m_summaryWidget);
 
-    m_treeView = new ResultsTreeView(m_outputWidget);
+    m_treeView = new ResultsTreeView(visualOutputWidget);
     m_treeView->setHeaderHidden(true);
     m_treeView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
     m_treeView->setContextMenuPolicy(Qt::CustomContextMenu);
+    pal = m_treeView->palette();
+    pal.setColor(QPalette::Base, pal.window().color());
+    m_treeView->setPalette(pal);
     m_model = new TestResultModel(this);
     m_filterModel = new TestResultFilterModel(m_model, this);
     m_filterModel->setDynamicSortFilter(true);
@@ -119,6 +124,19 @@ TestResultsPane::TestResultsPane(QObject *parent) :
 
     outputLayout->addWidget(Core::ItemViewFind::createSearchableWrapper(m_treeView));
 
+    m_textOutput = new QPlainTextEdit;
+    m_textOutput->setPalette(pal);
+    QFont font("monospace");
+    font.setStyleHint(QFont::TypeWriter);
+    m_textOutput->setFont(font);
+    m_textOutput->setWordWrapMode(QTextOption::WordWrap);
+    m_textOutput->setReadOnly(true);
+    m_outputWidget->addWidget(m_textOutput);
+
+    auto agg = new Aggregation::Aggregate;
+    agg->add(m_textOutput);
+    agg->add(new Core::BaseTextFind(m_textOutput));
+
     createToolButtons();
 
     connect(m_treeView, &Utils::TreeView::activated, this, &TestResultsPane::onItemActivated);
@@ -127,7 +145,7 @@ TestResultsPane::TestResultsPane(QObject *parent) :
     connect(m_treeView, &Utils::TreeView::customContextMenuRequested,
             this, &TestResultsPane::onCustomContextMenuRequested);
     connect(m_treeView, &ResultsTreeView::copyShortcutTriggered, [this] () {
-       onCopyItemTriggered(m_treeView->currentIndex());
+        onCopyItemTriggered(getTestResult(m_treeView->currentIndex()));
     });
     connect(m_model, &TestResultModel::requestExpansion, [this] (QModelIndex idx) {
         m_treeView->expand(m_filterModel->mapFromSource(idx));
@@ -138,15 +156,12 @@ TestResultsPane::TestResultsPane(QObject *parent) :
             this, &TestResultsPane::onTestRunFinished);
     connect(TestRunner::instance(), &TestRunner::testResultReady,
             this, &TestResultsPane::addTestResult);
-    connect(ProjectExplorer::ProjectExplorerPlugin::instance(),
-            &ProjectExplorer::ProjectExplorerPlugin::updateRunActions,
-            this, &TestResultsPane::updateRunActions);
 }
 
 void TestResultsPane::createToolButtons()
 {
     m_expandCollapse = new QToolButton(m_treeView);
-    m_expandCollapse->setIcon(Core::Icons::EXPAND_ALL_TOOLBAR.icon());
+    m_expandCollapse->setIcon(Utils::Icons::EXPAND_ALL_TOOLBAR.icon());
     m_expandCollapse->setToolTip(tr("Expand All"));
     m_expandCollapse->setCheckable(true);
     m_expandCollapse->setChecked(false);
@@ -158,28 +173,22 @@ void TestResultsPane::createToolButtons()
     });
 
     m_runAll = new QToolButton(m_treeView);
-    m_runAll->setIcon(Core::Icons::RUN_SMALL_TOOLBAR.icon());
-    m_runAll->setToolTip(tr("Run All Tests"));
-    m_runAll->setEnabled(false);
-    connect(m_runAll, &QToolButton::clicked, this, &TestResultsPane::onRunAllTriggered);
+    m_runAll->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_ALL_ID)->action());
 
     m_runSelected = new QToolButton(m_treeView);
-    Utils::Icon runSelectedIcon = Core::Icons::RUN_SMALL_TOOLBAR;
-    foreach (const Utils::IconMaskAndColor &maskAndColor, Icons::RUN_SELECTED_OVERLAY)
-        runSelectedIcon.append(maskAndColor);
-    m_runSelected->setIcon(runSelectedIcon.icon());
-    m_runSelected->setToolTip(tr("Run Selected Tests"));
-    m_runSelected->setEnabled(false);
-    connect(m_runSelected, &QToolButton::clicked, this, &TestResultsPane::onRunSelectedTriggered);
+    m_runSelected->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_SELECTED_ID)->action());
+
+    m_runFile = new QToolButton(m_treeView);
+    m_runFile->setDefaultAction(Core::ActionManager::command(Constants::ACTION_RUN_FILE_ID)->action());
 
     m_stopTestRun = new QToolButton(m_treeView);
-    m_stopTestRun->setIcon(Core::Icons::STOP_SMALL_TOOLBAR.icon());
+    m_stopTestRun->setIcon(Utils::Icons::STOP_SMALL_TOOLBAR.icon());
     m_stopTestRun->setToolTip(tr("Stop Test Run"));
     m_stopTestRun->setEnabled(false);
     connect(m_stopTestRun, &QToolButton::clicked, TestRunner::instance(), &TestRunner::requestStopTestRun);
 
     m_filterButton = new QToolButton(m_treeView);
-    m_filterButton->setIcon(Core::Icons::FILTER.icon());
+    m_filterButton->setIcon(Utils::Icons::FILTER.icon());
     m_filterButton->setToolTip(tr("Filter Test Results"));
     m_filterButton->setProperty("noArrow", true);
     m_filterButton->setAutoRaise(true);
@@ -188,21 +197,28 @@ void TestResultsPane::createToolButtons()
     initializeFilterMenu();
     connect(m_filterMenu, &QMenu::triggered, this, &TestResultsPane::filterMenuTriggered);
     m_filterButton->setMenu(m_filterMenu);
+    m_outputToggleButton = new QToolButton(m_treeView);
+    m_outputToggleButton->setIcon(Icons::TEXT_DISPLAY.icon());
+    m_outputToggleButton->setToolTip(tr("Switch Between Visual and Text Display"));
+    m_outputToggleButton->setEnabled(true);
+    connect(m_outputToggleButton, &QToolButton::clicked, this, &TestResultsPane::toggleOutputStyle);
 }
 
-static TestResultsPane *m_instance = 0;
+static TestResultsPane *s_instance = nullptr;
 
 TestResultsPane *TestResultsPane::instance()
 {
-    if (!m_instance)
-        m_instance = new TestResultsPane;
-    return m_instance;
+    if (!s_instance)
+        s_instance = new TestResultsPane;
+    return s_instance;
 }
 
 TestResultsPane::~TestResultsPane()
 {
     delete m_treeView;
-    m_instance = 0;
+    if (!m_outputWidget->parent())
+        delete m_outputWidget;
+    s_instance = nullptr;
 }
 
 void TestResultsPane::addTestResult(const TestResultPtr &result)
@@ -211,14 +227,18 @@ void TestResultsPane::addTestResult(const TestResultPtr &result)
     m_atEnd = scrollBar ? scrollBar->value() == scrollBar->maximum() : true;
 
     m_model->addTestResult(result, m_expandCollapse->isChecked());
-    if (!m_treeView->isVisible())
-        popup(Core::IOutputPane::NoModeSwitch);
     setIconBadgeNumber(m_model->resultTypeCount(Result::Fail)
+                       + m_model->resultTypeCount(Result::MessageFatal)
                        + m_model->resultTypeCount(Result::UnexpectedPass));
     flash();
     navigateStateChanged();
 }
 
+void TestResultsPane::addOutput(const QByteArray &output)
+{
+    m_textOutput->appendPlainText(QString::fromLatin1(output));
+}
+
 QWidget *TestResultsPane::outputWidget(QWidget *parent)
 {
     if (m_outputWidget) {
@@ -231,7 +251,8 @@ QWidget *TestResultsPane::outputWidget(QWidget *parent)
 
 QList<QWidget *> TestResultsPane::toolBarWidgets() const
 {
-    return { m_expandCollapse, m_runAll, m_runSelected, m_stopTestRun, m_filterButton };
+    return {m_expandCollapse, m_runAll, m_runSelected, m_runFile, m_stopTestRun,
+            m_outputToggleButton, m_filterButton};
 }
 
 QString TestResultsPane::displayName() const
@@ -247,30 +268,20 @@ int TestResultsPane::priorityInStatusBar() const
 void TestResultsPane::clearContents()
 {
     m_filterModel->clearTestResults();
+    if (auto delegate = qobject_cast<TestResultDelegate *>(m_treeView->itemDelegate()))
+        delegate->clearCache();
     setIconBadgeNumber(0);
     navigateStateChanged();
     m_summaryWidget->setVisible(false);
-    m_autoScroll = AutotestPlugin::instance()->settings()->autoScroll;
+    m_autoScroll = AutotestPlugin::settings()->autoScroll;
     connect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
             this, &TestResultsPane::onScrollBarRangeChanged, Qt::UniqueConnection);
+    m_textOutput->clear();
+    clearMarks();
 }
 
-void TestResultsPane::visibilityChanged(bool visible)
+void TestResultsPane::visibilityChanged(bool /*visible*/)
 {
-    if (visible == m_wasVisibleBefore)
-        return;
-    if (visible) {
-        connect(TestTreeModel::instance(), &TestTreeModel::testTreeModelChanged,
-                this, &TestResultsPane::updateRunActions);
-        // make sure run/run all are in correct state
-        updateRunActions();
-        TestTreeModel::instance()->enableParsing();
-    } else {
-        disconnect(TestTreeModel::instance(), &TestTreeModel::testTreeModelChanged,
-                   this, &TestResultsPane::updateRunActions);
-        TestTreeModel::instance()->disableParsing();
-    }
-    m_wasVisibleBefore = visible;
 }
 
 void TestResultsPane::setFocus()
@@ -396,19 +407,19 @@ void TestResultsPane::onRunAllTriggered()
 {
     TestRunner *runner = TestRunner::instance();
     runner->setSelectedTests(TestTreeModel::instance()->getAllTestCases());
-    runner->prepareToRunTests(TestRunner::Run);
+    runner->prepareToRunTests(TestRunMode::Run);
 }
 
 void TestResultsPane::onRunSelectedTriggered()
 {
     TestRunner *runner = TestRunner::instance();
     runner->setSelectedTests(TestTreeModel::instance()->getSelectedTests());
-    runner->prepareToRunTests(TestRunner::Run);
+    runner->prepareToRunTests(TestRunMode::Run);
 }
 
 void TestResultsPane::initializeFilterMenu()
 {
-    const bool omitIntern = AutotestPlugin::instance()->settings()->omitInternalMssg;
+    const bool omitIntern = AutotestPlugin::settings()->omitInternalMssg;
     // FilterModel has all messages enabled by default
     if (omitIntern)
         m_filterModel->toggleTestResultType(Result::MessageInternal);
@@ -423,7 +434,7 @@ void TestResultsPane::initializeFilterMenu()
     textAndType.insert(Result::MessageDebug, tr("Debug Messages"));
     textAndType.insert(Result::MessageWarn, tr("Warning Messages"));
     textAndType.insert(Result::MessageInternal, tr("Internal Messages"));
-    foreach (Result::Type result, textAndType.keys()) {
+    for (Result::Type result : textAndType.keys()) {
         QAction *action = new QAction(m_filterMenu);
         action->setText(textAndType.value(result));
         action->setCheckable(true);
@@ -441,38 +452,34 @@ void TestResultsPane::initializeFilterMenu()
 
 void TestResultsPane::updateSummaryLabel()
 {
-    QString labelText = QString::fromLatin1("<p>Test summary:&nbsp;&nbsp; %1 %2, %3 %4")
+    QString labelText = QString("<p>Test summary:&nbsp;&nbsp; %1 %2, %3 %4")
             .arg(QString::number(m_model->resultTypeCount(Result::Pass)), tr("passes"),
                  QString::number(m_model->resultTypeCount(Result::Fail)), tr("fails"));
     int count = m_model->resultTypeCount(Result::UnexpectedPass);
     if (count)
-        labelText.append(QString::fromLatin1(", %1 %2")
-                         .arg(QString::number(count), tr("unexpected passes")));
+        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("unexpected passes")));
     count = m_model->resultTypeCount(Result::ExpectedFail);
     if (count)
-        labelText.append(QString::fromLatin1(", %1 %2")
-                         .arg(QString::number(count), tr("expected fails")));
+        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("expected fails")));
     count = m_model->resultTypeCount(Result::MessageFatal);
     if (count)
-        labelText.append(QString::fromLatin1(", %1 %2")
-                         .arg(QString::number(count), tr("fatals")));
+        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("fatals")));
     count = m_model->resultTypeCount(Result::BlacklistedFail)
             + m_model->resultTypeCount(Result::BlacklistedPass);
     if (count)
-        labelText.append(QString::fromLatin1(", %1 %2")
-                         .arg(QString::number(count), tr("blacklisted")));
+        labelText.append(QString(", %1 %2").arg(QString::number(count), tr("blacklisted")));
 
     count = m_model->disabledTests();
     if (count)
         labelText.append(tr(", %1 disabled").arg(count));
 
-    labelText.append(QLatin1String(".</p>"));
+    labelText.append(".</p>");
     m_summaryLabel->setText(labelText);
 }
 
 void TestResultsPane::enableAllFilter()
 {
-    foreach (QAction *action, m_filterMenu->actions()) {
+    for (QAction *action : m_filterMenu->actions()) {
         if (action->isCheckable())
             action->setChecked(true);
     }
@@ -489,8 +496,7 @@ void TestResultsPane::onTestRunStarted()
 {
     m_testRunning = true;
     m_stopTestRun->setEnabled(true);
-    m_runAll->setEnabled(false);
-    m_runSelected->setEnabled(false);
+    AutotestPlugin::updateMenuItemsEnabledState();
     m_summaryWidget->setVisible(false);
 }
 
@@ -498,13 +504,16 @@ void TestResultsPane::onTestRunFinished()
 {
     m_testRunning = false;
     m_stopTestRun->setEnabled(false);
-    m_runAll->setEnabled(true);
-    m_runSelected->setEnabled(true);
+
+    AutotestPlugin::updateMenuItemsEnabledState();
     updateSummaryLabel();
     m_summaryWidget->setVisible(true);
     m_model->removeCurrentTestMessage();
     disconnect(m_treeView->verticalScrollBar(), &QScrollBar::rangeChanged,
                this, &TestResultsPane::onScrollBarRangeChanged);
+    if (!m_treeView->isVisible())
+        popup(Core::IOutputPane::NoModeSwitch);
+    createMarks();
 }
 
 void TestResultsPane::onScrollBarRangeChanged(int, int max)
@@ -513,26 +522,16 @@ void TestResultsPane::onScrollBarRangeChanged(int, int max)
         m_treeView->verticalScrollBar()->setValue(max);
 }
 
-void TestResultsPane::updateRunActions()
-{
-    QString whyNot;
-    TestTreeModel *model = TestTreeModel::instance();
-    const bool enable = !m_testRunning && !model->parser()->isParsing() && model->hasTests()
-            && ProjectExplorer::ProjectExplorerPlugin::canRunStartupProject(
-                ProjectExplorer::Constants::NORMAL_RUN_MODE, &whyNot);
-    m_runAll->setEnabled(enable);
-    m_runSelected->setEnabled(enable);
-}
-
 void TestResultsPane::onCustomContextMenuRequested(const QPoint &pos)
 {
     const bool resultsAvailable = m_filterModel->hasResults();
     const bool enabled = !m_testRunning && resultsAvailable;
-    const QModelIndex clicked = m_treeView->indexAt(pos);
+    const TestResult *clicked = getTestResult(m_treeView->indexAt(pos));
     QMenu menu;
+
     QAction *action = new QAction(tr("Copy"), &menu);
     action->setShortcut(QKeySequence(QKeySequence::Copy));
-    action->setEnabled(resultsAvailable);
+    action->setEnabled(resultsAvailable && clicked);
     connect(action, &QAction::triggered, [this, clicked] () {
        onCopyItemTriggered(clicked);
     });
@@ -548,12 +547,37 @@ void TestResultsPane::onCustomContextMenuRequested(const QPoint &pos)
     connect(action, &QAction::triggered, this, &TestResultsPane::onSaveWholeTriggered);
     menu.addAction(action);
 
+    const auto correlatingItem = clicked ? clicked->findTestTreeItem() : nullptr;
+    action = new QAction(tr("Run This Test"), &menu);
+    action->setEnabled(correlatingItem && correlatingItem->canProvideTestConfiguration());
+    connect(action, &QAction::triggered, this, [this, clicked] {
+        onRunThisTestTriggered(TestRunMode::Run, clicked);
+    });
+    menu.addAction(action);
+
+    action = new QAction(tr("Debug This Test"), &menu);
+    action->setEnabled(correlatingItem && correlatingItem->canProvideDebugConfiguration());
+    connect(action, &QAction::triggered, this, [this, clicked] {
+        onRunThisTestTriggered(TestRunMode::Debug, clicked);
+    });
+    menu.addAction(action);
+
     menu.exec(m_treeView->mapToGlobal(pos));
 }
 
-void TestResultsPane::onCopyItemTriggered(const QModelIndex &idx)
+const TestResult *TestResultsPane::getTestResult(const QModelIndex &idx)
 {
+    if (!idx.isValid())
+        return nullptr;
+
     const TestResult *result = m_filterModel->testResult(idx);
+    QTC_CHECK(result);
+
+    return result;
+}
+
+void TestResultsPane::onCopyItemTriggered(const TestResult *result)
+{
     QTC_ASSERT(result, return);
     QApplication::clipboard()->setText(result->outputString(true));
 }
@@ -566,7 +590,7 @@ void TestResultsPane::onCopyWholeTriggered()
 void TestResultsPane::onSaveWholeTriggered()
 {
     const QString fileName = QFileDialog::getSaveFileName(Core::ICore::dialogParent(),
-                                                          tr("Save Output To..."));
+                                                          tr("Save Output To"));
     if (fileName.isEmpty())
         return;
 
@@ -578,6 +602,24 @@ void TestResultsPane::onSaveWholeTriggered()
     }
 }
 
+void TestResultsPane::onRunThisTestTriggered(TestRunMode runMode, const TestResult *result)
+{
+    QTC_ASSERT(result, return);
+
+    const TestTreeItem *item = result->findTestTreeItem();
+
+    if (item)
+        TestRunner::instance()->runTest(runMode, item);
+}
+
+void TestResultsPane::toggleOutputStyle()
+{
+    const bool displayText = m_outputWidget->currentIndex() == 0;
+    m_outputWidget->setCurrentIndex(displayText ? 1 : 0);
+    m_outputToggleButton->setIcon(displayText ? Icons::VISUAL_DISPLAY.icon()
+                                              : Icons::TEXT_DISPLAY.icon());
+}
+
 // helper for onCopyWholeTriggered() and onSaveWholeTriggered()
 QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
 {
@@ -586,12 +628,50 @@ QString TestResultsPane::getWholeOutput(const QModelIndex &parent)
         QModelIndex current = m_model->index(row, 0, parent);
         const TestResult *result = m_model->testResult(current);
         QTC_ASSERT(result, continue);
-        output.append(TestResult::resultToString(result->result())).append(QLatin1Char('\t'));
-        output.append(result->outputString(true)).append(QLatin1Char('\n'));
+        output.append(TestResult::resultToString(result->result())).append('\t');
+        output.append(result->outputString(true)).append('\n');
         output.append(getWholeOutput(current));
     }
     return output;
 }
 
+void TestResultsPane::createMarks(const QModelIndex &parent)
+{
+    for (int row = 0, count = m_model->rowCount(parent); row < count; ++row) {
+        const QModelIndex index = m_model->index(row, 0, parent);
+        const TestResult *result = m_model->testResult(index);
+        QTC_ASSERT(result, continue);
+
+        if (m_model->hasChildren(index))
+            createMarks(index);
+
+        const QVector<Result::Type> interested{Result::Fail, Result::UnexpectedPass};
+        if (interested.contains(result->result())) {
+            const Utils::FileName fileName = Utils::FileName::fromString(result->fileName());
+            TestEditorMark *mark = new TestEditorMark(index, fileName, result->line());
+            mark->setIcon(index.data(Qt::DecorationRole).value<QIcon>());
+            mark->setColor(Utils::Theme::OutputPanes_TestFailTextColor);
+            mark->setPriority(TextEditor::TextMark::NormalPriority);
+            mark->setToolTip(result->description());
+            m_marks << mark;
+        }
+    }
+}
+
+void TestResultsPane::clearMarks()
+{
+    qDeleteAll(m_marks);
+    m_marks.clear();
+}
+
+void TestResultsPane::showTestResult(const QModelIndex &index)
+{
+    QModelIndex mapped = m_filterModel->mapFromSource(index);
+    if (mapped.isValid()) {
+        popup(Core::IOutputPane::NoModeSwitch);
+        m_treeView->setCurrentIndex(mapped);
+    }
+}
+
 } // namespace Internal
 } // namespace Autotest
