@@ -125,7 +125,7 @@ Location::Location(const StackFrame &frame, bool marker)
 }
 
 
-LocationMark::LocationMark(DebuggerEngine *engine, const QString &file, int line)
+LocationMark::LocationMark(DebuggerEngine *engine, const FileName &file, int line)
     : TextMark(file, line, Constants::TEXT_MARK_CATEGORY_LOCATION), m_engine(engine)
 {
     setIcon(Icons::LOCATION.icon());
@@ -486,6 +486,7 @@ void DebuggerEngine::start()
         d->m_runTool->runControl()->setApplicationProcessHandle(d->m_inferiorPid);
 
     action(OperateByInstruction)->setEnabled(hasCapability(DisassemblerCapability));
+    action(OperateByInstruction)->setChecked(boolSetting(OperateByInstruction));
 
     QTC_ASSERT(state() == DebuggerNotReady || state() == DebuggerFinished,
          qDebug() << state());
@@ -534,8 +535,10 @@ void DebuggerEngine::gotoLocation(const Location &loc)
     const QString file = QDir::cleanPath(loc.fileName());
     const int line = loc.lineNumber();
     bool newEditor = false;
-    IEditor *editor = EditorManager::openEditor(file, Id(),
-                                                EditorManager::IgnoreNavigationHistory, &newEditor);
+    IEditor *editor = EditorManager::openEditor(
+                file, Id(),
+                EditorManager::IgnoreNavigationHistory | EditorManager::DoNotSwitchToDesignMode,
+                &newEditor);
     QTC_ASSERT(editor, return); // Unreadable file?
 
     editor->gotoLine(line, 0, !boolSetting(StationaryEditorWhileStepping));
@@ -544,7 +547,7 @@ void DebuggerEngine::gotoLocation(const Location &loc)
         editor->document()->setProperty(Constants::OPENED_BY_DEBUGGER, true);
 
     if (loc.needsMarker())
-        d->m_locationMark.reset(new LocationMark(this, file, line));
+        d->m_locationMark.reset(new LocationMark(this, FileName::fromString(file), line));
 }
 
 const DebuggerRunParameters &DebuggerEngine::runParameters() const
@@ -650,12 +653,9 @@ void DebuggerEngine::notifyEngineSetupOk()
     d->m_progress.setProgressValue(250);
     QTC_ASSERT(state() == EngineSetupRequested, qDebug() << this << state());
     setState(EngineSetupOk);
-    if (isMasterEngine() && runTool()) {
-        runTool()->aboutToNotifyInferiorSetupOk(); // FIXME: Remove, only used for Android.
-        runTool()->reportStarted();
-    }
-
     if (isMasterEngine()) {
+        if (runTool())
+            runTool()->reportStarted();
         // Slaves will get called setupSlaveInferior() below.
         setState(EngineRunRequested);
         showMessage("CALL: RUN ENGINE");
@@ -1421,6 +1421,11 @@ void DebuggerEngine::changeBreakpoint(Breakpoint bp)
     QTC_CHECK(false);
 }
 
+void DebuggerEngine::enableSubBreakpoint(const QString &, bool)
+{
+    QTC_CHECK(false);
+}
+
 void DebuggerEngine::assignValueInDebugger(WatchItem *,
     const QString &, const QVariant &)
 {
