@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,6 +28,7 @@
 #include "debuggeractions.h"
 #include "debuggercore.h"
 #include "debuggerengine.h"
+#include "debuggericons.h"
 
 #include <QDebug>
 #include <QTime>
@@ -47,6 +43,9 @@
 #include <QToolButton>
 
 #include <aggregation/aggregate.h>
+
+#include <app/app_version.h>
+
 #include <coreplugin/findplaceholder.h>
 #include <coreplugin/minisplitter.h>
 #include <coreplugin/find/basetextfind.h>
@@ -54,6 +53,7 @@
 #include <utils/savedaction.h>
 #include <utils/fancylineedit.h>
 #include <utils/fileutils.h>
+#include <utils/theme/theme.h>
 
 namespace Debugger {
 namespace Internal {
@@ -74,26 +74,28 @@ public:
 private:
     void highlightBlock(const QString &text)
     {
+        using Utils::Theme;
         QTextCharFormat format;
+        Theme *theme = Utils::creatorTheme();
         switch (LogWindow::channelForChar(text.isEmpty() ? QChar() : text.at(0))) {
             case LogInput:
-                format.setForeground(Qt::blue);
+                format.setForeground(theme->color(Theme::Debugger_LogWindow_LogInput));
                 setFormat(1, text.size(), format);
                 break;
             case LogStatus:
-                format.setForeground(Qt::darkGreen);
+                format.setForeground(theme->color(Theme::Debugger_LogWindow_LogStatus));
                 setFormat(1, text.size(), format);
                 break;
             case LogWarning:
-                format.setForeground(Qt::darkYellow);
+                format.setForeground(theme->color(Theme::OutputPanes_WarningMessageTextColor));
                 setFormat(1, text.size(), format);
                 break;
             case LogError:
-                format.setForeground(Qt::red);
+                format.setForeground(theme->color(Theme::OutputPanes_ErrorMessageTextColor));
                 setFormat(1, text.size(), format);
                 break;
             case LogTime:
-                format.setForeground(Qt::darkRed);
+                format.setForeground(theme->color(Theme::Debugger_LogWindow_LogTime));
                 setFormat(1, text.size(), format);
                 break;
             default:
@@ -125,9 +127,11 @@ public:
 private:
     void highlightBlock(const QString &text)
     {
+        using Utils::Theme;
+        Theme *theme = Utils::creatorTheme();
         if (text.size() > 3 && text.at(2) == QLatin1Char(':')) {
             QTextCharFormat format;
-            format.setForeground(Qt::darkRed);
+            format.setForeground(theme->color(Theme::Debugger_LogWindow_LogTime));
             setFormat(1, text.size(), format);
         }
     }
@@ -145,27 +149,27 @@ class DebuggerPane : public QPlainTextEdit
     Q_OBJECT
 
 public:
-    DebuggerPane(QWidget *parent)
+    DebuggerPane(LogWindow *parent)
         : QPlainTextEdit(parent)
     {
         setFrameStyle(QFrame::NoFrame);
         m_clearContentsAction = new QAction(this);
         m_clearContentsAction->setText(tr("Clear Contents"));
         m_clearContentsAction->setEnabled(true);
-        connect(m_clearContentsAction, SIGNAL(triggered(bool)),
-            parent, SLOT(clearContents()));
+        connect(m_clearContentsAction, &QAction::triggered,
+                parent, &LogWindow::clearContents);
 
         m_saveContentsAction = new QAction(this);
         m_saveContentsAction->setText(tr("Save Contents"));
         m_saveContentsAction->setEnabled(true);
-        connect(m_saveContentsAction, SIGNAL(triggered()),
-            this, SLOT(saveContents()));
+        connect(m_saveContentsAction, &QAction::triggered,
+                this, &DebuggerPane::saveContents);
 
         m_reloadDebuggingHelpersAction = new QAction(this);
         m_reloadDebuggingHelpersAction->setText(tr("Reload Debugging Helpers"));
         m_reloadDebuggingHelpersAction->setEnabled(true);
-        connect(m_reloadDebuggingHelpersAction, SIGNAL(triggered()),
-            this, SLOT(reloadDebuggingHelpers()));
+        connect(m_reloadDebuggingHelpersAction, &QAction::triggered,
+                this, &DebuggerPane::reloadDebuggingHelpers);
     }
 
     void contextMenuEvent(QContextMenuEvent *ev)
@@ -173,11 +177,10 @@ public:
         QMenu *menu = createStandardContextMenu();
         menu->addAction(m_clearContentsAction);
         menu->addAction(m_saveContentsAction); // X11 clipboard is unreliable for long texts
-        menu->addAction(debuggerCore()->action(LogTimeStamps));
-        menu->addAction(debuggerCore()->action(VerboseLog));
+        menu->addAction(action(LogTimeStamps));
         menu->addAction(m_reloadDebuggingHelpersAction);
         menu->addSeparator();
-        menu->addAction(debuggerCore()->action(SettingsDialog));
+        menu->addAction(action(SettingsDialog));
         menu->exec(ev->globalPos());
         delete menu;
     }
@@ -209,11 +212,10 @@ public:
         setUndoRedoEnabled(true);
     }
 
-private slots:
+private:
     void saveContents();
     void reloadDebuggingHelpers();
 
-private:
     QAction *m_clearContentsAction;
     QAction *m_saveContentsAction;
     QAction *m_reloadDebuggingHelpersAction;
@@ -226,7 +228,7 @@ void DebuggerPane::saveContents()
 
 void DebuggerPane::reloadDebuggingHelpers()
 {
-    debuggerCore()->currentEngine()->reloadDebuggingHelpers();
+    currentEngine()->reloadDebuggingHelpers();
 }
 
 /////////////////////////////////////////////////////////////////////
@@ -239,7 +241,7 @@ class InputPane : public DebuggerPane
 {
     Q_OBJECT
 public:
-    InputPane(QWidget *parent)
+    InputPane(LogWindow *parent)
         : DebuggerPane(parent)
     {
         (void) new InputHighlighter(this);
@@ -305,13 +307,12 @@ class CombinedPane : public DebuggerPane
 {
     Q_OBJECT
 public:
-    CombinedPane(QWidget *parent)
+    CombinedPane(LogWindow *parent)
         : DebuggerPane(parent)
     {
         (void) new OutputHighlighter(this);
     }
 
-public slots:
     void gotoResult(int i)
     {
         QString needle = QString::number(i) + QLatin1Char('^');
@@ -349,12 +350,12 @@ public slots:
 LogWindow::LogWindow(QWidget *parent)
   : QWidget(parent)
 {
-    setWindowTitle(tr("Debugger Log"));
+    setWindowTitle(tr("Debugger &Log"));
     setObjectName(QLatin1String("Log"));
 
     m_ignoreNextInputEcho = false;
 
-    QSplitter *m_splitter = new Core::MiniSplitter(Qt::Horizontal);
+    auto m_splitter = new Core::MiniSplitter(Qt::Horizontal);
     m_splitter->setParent(this);
 
     // Mixed input/output.
@@ -374,25 +375,25 @@ LogWindow::LogWindow(QWidget *parent)
     m_commandEdit->setFrame(false);
     m_commandEdit->setHistoryCompleter(QLatin1String("DebuggerInput"));
 
-    QToolButton *repeatButton = new QToolButton(this);
-    repeatButton->setIcon(QIcon(QLatin1String(":/debugger/images/debugger_stepover_small.png")));
-    repeatButton->setIconSize(QSize(12, 12));
+    auto repeatButton = new QToolButton(this);
+    repeatButton->setIcon(Icons::STEP_OVER.icon());
+    repeatButton->setFixedSize(QSize(18, 18));
     repeatButton->setToolTip(tr("Repeat last command for debug reasons."));
 
-    QHBoxLayout *commandBox = new QHBoxLayout;
+    auto commandBox = new QHBoxLayout;
     commandBox->addWidget(repeatButton);
     commandBox->addWidget(new QLabel(tr("Command:"), this));
     commandBox->addWidget(m_commandEdit);
     commandBox->setMargin(2);
     commandBox->setSpacing(6);
 
-    QVBoxLayout *leftBox = new QVBoxLayout;
+    auto leftBox = new QVBoxLayout;
     leftBox->addWidget(m_inputText);
     leftBox->addItem(commandBox);
     leftBox->setMargin(0);
     leftBox->setSpacing(0);
 
-    QWidget *leftDummy = new QWidget;
+    auto leftDummy = new QWidget;
     leftDummy->setLayout(leftBox);
 
     m_splitter->addWidget(leftDummy);
@@ -400,14 +401,14 @@ LogWindow::LogWindow(QWidget *parent)
     m_splitter->setStretchFactor(0, 1);
     m_splitter->setStretchFactor(1, 3);
 
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    auto layout = new QVBoxLayout(this);
     layout->setMargin(0);
     layout->setSpacing(0);
     layout->addWidget(m_splitter);
     layout->addWidget(new Core::FindToolBarPlaceHolder(this));
     setLayout(layout);
 
-    Aggregation::Aggregate *aggregate = new Aggregation::Aggregate;
+    auto aggregate = new Aggregation::Aggregate;
     aggregate->add(m_combinedText);
     aggregate->add(new Core::BaseTextFind(m_combinedText));
 
@@ -415,37 +416,50 @@ LogWindow::LogWindow(QWidget *parent)
     aggregate->add(m_inputText);
     aggregate->add(new Core::BaseTextFind(m_inputText));
 
-    connect(m_inputText, SIGNAL(statusMessageRequested(QString,int)),
-        SIGNAL(statusMessageRequested(QString,int)));
-    connect(m_inputText, SIGNAL(commandSelected(int)),
-        m_combinedText, SLOT(gotoResult(int)));
-    connect(m_commandEdit, SIGNAL(returnPressed()),
-        SLOT(sendCommand()));
-    connect(m_inputText, SIGNAL(executeLineRequested()),
-        SLOT(executeLine()));
-    connect(repeatButton, SIGNAL(clicked()),
-        SLOT(repeatLastCommand()));
+    connect(m_inputText, &InputPane::statusMessageRequested,
+            this, &LogWindow::statusMessageRequested);
+    connect(m_inputText, &InputPane::commandSelected,
+            m_combinedText, &CombinedPane::gotoResult);
+    connect(m_commandEdit, &QLineEdit::returnPressed,
+            this, &LogWindow::sendCommand);
+    connect(m_inputText, &InputPane::executeLineRequested,
+            this, &LogWindow::executeLine);
+    connect(repeatButton, &QAbstractButton::clicked,
+            this, &LogWindow::repeatLastCommand);
 
-    connect(&m_outputTimer, SIGNAL(timeout()), SLOT(doOutput()));
+    connect(&m_outputTimer, &QTimer::timeout,
+            this, &LogWindow::doOutput);
 
     setMinimumHeight(60);
+
+    showOutput(LogWarning,
+        tr("Note: This log contains possibly confidential information about your machine, "
+           "environment variables, in-memory data of the processes you are debugging, and more. "
+           "It is never transferred over the internet by %1, and only stored "
+           "to disk if you manually use the respective option from the context menu, or through "
+           "mechanisms that are not under the control of %1's Debugger plugin, "
+           "for instance in swap files, or other plugins you might use.\n"
+           "You may be asked to share the contents of this log when reporting bugs related "
+           "to debugger operation. In this case, make sure your submission does not "
+           "contain data you do not want to or you are not allowed to share.\n\n")
+               .arg(Core::Constants::IDE_DISPLAY_NAME));
 }
 
 void LogWindow::executeLine()
 {
     m_ignoreNextInputEcho = true;
-    debuggerCore()->currentEngine()->
+    currentEngine()->
         executeDebuggerCommand(m_inputText->textCursor().block().text(), CppLanguage);
 }
 
 void LogWindow::repeatLastCommand()
 {
-    debuggerCore()->currentEngine()->debugLastCommand();
+    currentEngine()->debugLastCommand();
 }
 
 void LogWindow::sendCommand()
 {
-    DebuggerEngine *engine = debuggerCore()->currentEngine();
+    DebuggerEngine *engine = currentEngine();
     if (engine->acceptsDebuggerCommands())
         engine->executeDebuggerCommand(m_commandEdit->text(), CppLanguage);
     else
@@ -463,7 +477,7 @@ void LogWindow::showOutput(int channel, const QString &output)
     QString out;
     out.reserve(output.size() + 1000);
 
-    if (output.at(0) != QLatin1Char('~') && debuggerCore()->boolSetting(LogTimeStamps)) {
+    if (output.at(0) != QLatin1Char('~') && boolSetting(LogTimeStamps)) {
         out.append(charForChannel(LogTime));
         out.append(logTimeStamp());
         out.append(nchar);
@@ -522,7 +536,7 @@ void LogWindow::showInput(int channel, const QString &input)
         m_inputText->setTextCursor(cursor);
         return;
     }
-    if (debuggerCore()->boolSetting(LogTimeStamps))
+    if (boolSetting(LogTimeStamps))
         m_inputText->append(logTimeStamp());
     m_inputText->append(input);
     QTextCursor cursor = m_inputText->textCursor();
