@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef PARSETREENODES_H
-#define PARSETREENODES_H
+#pragma once
 
 #include "globalparsestate.h"
 
@@ -291,7 +290,7 @@ private:
     void parse();
     QByteArray description() const;
 
-    Type m_type;
+    Type m_type = VendorType;
 };
 
 class ExprPrimaryNode : public ParseTreeNode
@@ -829,5 +828,3 @@ private:
 
 } // namespace Internal
 } // namespace Debugger
-
-#endif // PARSETREENODES_H
