@@ -214,10 +214,10 @@ def __handleColorTips__(colTip, expectedColor, alternativeColor):
         return value
 
     cmp = QColor()
-    cmp.setNamedColor(expectedColor)
+    cmp.setNamedColor(QString(expectedColor))
     if alternativeColor:
         alt = QColor()
-        alt.setNamedColor(alternativeColor)
+        alt.setNamedColor(QString(alternativeColor))
     if cmp.alpha() != 255 or alternativeColor and alt.alpha() != 255:
         test.warning("Cannot handle transparent colors - cancelling this verification")
         return
@@ -267,7 +267,7 @@ def getEditorForFileSuffix(curFile, treeViewSyntax=False):
     if treeViewSyntax:
         expected = simpleFileName(curFile)
     mainWindow = waitForObject(":Qt Creator_Core::Internal::MainWindow")
-    if not waitFor("expected in str(mainWindow.windowTitle)", 5000):
+    if not waitFor("str(mainWindow.windowTitle).startswith(expected + ' ')", 5000):
         test.fatal("Window title (%s) did not switch to expected file (%s)."
                    % (str(mainWindow.windowTitle), expected))
     try:
@@ -333,7 +333,7 @@ def validateSearchResult(expectedCount):
             resultTreeView.scrollTo(chIndex)
             text = str(chIndex.data()).rstrip('\r')
             rect = resultTreeView.visualRect(chIndex)
-            doubleClick(resultTreeView, rect.x+5, rect.y+5, 0, Qt.LeftButton)
+            doubleClick(resultTreeView, rect.x+50, rect.y+5, 0, Qt.LeftButton)
             editor = getEditorForFileSuffix(itemText)
             if not waitFor("lineUnderCursor(editor) == text", 2000):
                 test.warning("Jumping to search result '%s' is pretty slow." % text)
@@ -344,7 +344,10 @@ def validateSearchResult(expectedCount):
 def invokeContextMenuItem(editorArea, command1, command2 = None):
     ctxtMenu = openContextMenuOnTextCursorPosition(editorArea)
     snooze(1)
-    activateItem(waitForObjectItem(objectMap.realName(ctxtMenu), command1, 2000))
+    item1 = waitForObjectItem(objectMap.realName(ctxtMenu), command1, 2000)
+    if command2 and platform.system() == 'Darwin':
+        mouseMove(item1)
+    activateItem(item1)
     if command2:
         activateItem(waitForObjectItem("{title='%s' type='QMenu' visible='1' window=%s}"
                                        % (command1, objectMap.realName(ctxtMenu)), command2, 2000))
@@ -359,6 +362,7 @@ def invokeFindUsage(editor, line, typeOperation, n=1):
         return False
     for i in range(n):
         type(editor, typeOperation)
+    snooze(1)
     invokeContextMenuItem(editor, "Find Usages")
     return True
 
@@ -377,14 +381,14 @@ def openDocument(treeElement):
         except:
             treeElement = addBranchWildcardToRoot(treeElement)
             item = waitForObjectItem(navigator, treeElement)
+        expected = str(item.text).split("/")[-1]
         for _ in range(2):
             # Expands items as needed what might make scrollbars appear.
             # These might cover the item to click.
             # In this case, do it again to hit the item then.
             doubleClickItem(navigator, treeElement, 5, 5, 0, Qt.LeftButton)
             mainWindow = waitForObject(":Qt Creator_Core::Internal::MainWindow")
-            expected = str(item.text).split("/")[-1]
-            if waitFor("expected in str(mainWindow.windowTitle)", 5000):
+            if waitFor("str(mainWindow.windowTitle).startswith(expected + ' ')", 5000):
                 return True
         test.log("Expected file (%s) was not being opened in openDocument()" % expected)
         return False
@@ -420,3 +424,14 @@ def replaceLine(fileSpec, oldLine, newLine):
         type(editor, "<Backspace>")
     type(editor, newLine)
     return True
+
+def addTestableCodeAfterLine(editorObject, line, newCodeLines):
+    if not placeCursorToLine(editorObject, line):
+        return False
+    type(editorObject, "<Return>")
+    typeLines(editorObject, newCodeLines)
+    return True
+
+def saveAndExit():
+    invokeMenuItem("File", "Save All")
+    invokeMenuItem("File", "Exit")
